#### Analysis of mammalian body mass evolution in Puttick et al. 2020 ####

## Load motmot.
library(motmot)

## Source mammaliaforme data & tree from the package.
data(mammals)
attach(mammals)

## Remove missing data, match data names to tree names, and take the columns 'mean' and 'sem' measurements from 'mammal.mass' matrix
trait.phy <- sortTraitData(phy = mammal.phy, y = as.matrix(mammal.mass), data.name = c("mean", "sem"), log.trait = FALSE)
phy <- trait.phy$phy
y <- as.matrix(trait.phy$trait[,1])
errors <- as.numeric(trait.phy$trait[,2])

## PLot the phylogent and data using traitData.plot (not run).
#  traitData.plot(y = y, phy = phy, col.label = "navy", cex.axis = 1, cex.plot = 1.1, mgp = c(0, 0.3, 0), 
#    tck = -0.01 , at = log(c(1, 10, 1000, 10000, 100000, 1000000, 10000000, 100000000)),
#    labels = c(1, 10, 1000, 10000, 100000, 1000000, 10000000, 100000000), las = 2, 
#    axis.text = "Mammal mass (grams, log scale)", axis.text.line = 2.4, offset.bars = 1.03)
#  nodelabels("Placentalia", 223, frame = "n", adj = c(1.05, 1.2), cex = 1)

	
## models fit using transformPhylo.ML

## Brownian motion
bm.model <- transformPhylo.ML(y = y, phy = phy, model = "BM", meserr = errors)

## Ornstein-Uhlenbeck
ou.model <- transformPhylo.ML(y = y, phy = phy, model = "OU", meserr = errors)

## Early Burst
eb.model <- transformPhylo.ML(y = y, phy = phy, model = "ACDC", upperBound=-1e-6, meserr = errors)

## timeSlice with a split at 66 Ma
time.slice.66.model <- transformPhylo.ML(y = y, phy = phy, model = "timeSlice", splitTime = 66, meserr = errors)
time.slice.66.model.AICc <- plot.timeSlice.ML(time.slice.66.model, cutoff = 0, phylo.plot = FALSE)$Rates["AICc"]

## timeSlice with a split at 66 Ma and an additional split between 56
time.slice.66.model.multi <- transformPhylo.ML(y = y, phy = phy, model = "timeSlice", splitTime = c(66, 56), meserr = errors)
time.slice.66.model.multi.AICc <- plot.timeSlice.ML(time.slice.66.model.multi, cutoff = 0, phylo.plot = FALSE)$Rates["AICc"]

## timeSlice naive model	
time.slice.66.model_naive <- transformPhylo.ML(y = y, phy = phy, model = "timeSlice", nSplits = 2, boundaryAge = c(50, 20), meserr = errors, testAge = 1)
  ## Summarise naive results using model averaging (not run).
  # plot.timeSlice.ML(time.slice.66.model_naive, model.average = TRUE)
time.slice.66.model_naive.AICc <- plot.timeSlice.ML(time.slice.66.model_naive, cutoff = 0, phylo.plot = FALSE)$Rates["AICc"]

## release model, OU -> BM at KPg 66 Ma
release.model <- transformPhylo.ML(y = y, phy = phy, model = "modeslice", mode.order = c("ou", "bm"), splitTime = 66, meserr = errors)

## release and radiate model, OU -> BM (+ rate change) at KPg 66 Ma
release.radiate.model <- transformPhylo.ML(y = y, phy = phy, model = "modeslice", mode.order=c("ou", "bm"), splitTime = 66, meserr = errors, rate.var = TRUE)

## release, radiate, and recapture model
## OU -> BM (+ rate change) at KPg 66 Ma, BM -> OU at 53 Ma
release.radiate.recapture.model <- transformPhylo.ML(y = y, phy = phy, model = "modeslice", mode.order=c("ou", "bm", "ou"),
  splitTime=c(66, 56), meserr = errors, rate.var = TRUE)
	
## relative fit of all models tested using AICc, delta AICc, and AICc weights
aicc <- c(bm.model$AICc, ou.model$AICc, eb.model$AICc, time.slice.66.model.AICc, time.slice.66.model.multi.AICc,
  time.slice.66.model_naive.AICc, release.model$AICc, release.radiate.model$AICc, release.radiate.recapture.model$AICc)
aicc.delta <- aicc - min(aicc)
rel.lik <- exp(-0.5 * aicc.delta)
aicc.wt <- signif(rel.lik / sum(rel.lik), 3)

## Release radiate and recapture model is the best-fitting model
output.models <- matrix(cbind(aicc, aicc.delta, aicc.wt), ncol = 3,
  dimnames = list(c("BM", "OU", "EB", "time.slice.66.Ma", "time.slice.66.53.Ma", "time.slice.all.shifts", 
    "release.model", "release.radiate", "release.radiate.recapture"), c("AICc", "delta.AICc", "AICc.weights")))
    
    
    
    
    
    
dev.new(height=3.8, width=8)
	par(mar = c(1.2, 1.8, 5, 0.2))
	bp <- barplot(output.models[, 3], xaxt = "n", space = 0.4, border = "white", col = "white", yaxt = "n",
	  ylim = c(0, 1), mgp = c(0, 0.2, 0))
	axis(2, at = seq(0, 1, 0.2), las = 2, mgp = c(-1, -0.6, -1), tck = -0.01)
	
	lines(c(0.25 , 12.6), c(0, 0), col = "#00000010", xpd = TRUE)
	lines(c(0.25 , 12.6), c(0.2, 0.2), col = "#00000010", xpd = TRUE)
    lines(c(0.25 , 12.6), c(0.4, 0.4), col = "#00000010", xpd = TRUE)
	lines(c(0.25 , 12.6), c(0.6, 0.6), col = "#00000010", xpd = TRUE)
	lines(c(0.25 , 12.6), c(0.8, 0.8), col = "#00000010", xpd = TRUE)
	lines(c(0.25 , 12.6), c(1, 1), col = "#00000010", xpd = TRUE)

	bp <- barplot(output.models[, 3], xaxt = "n", space = 0.4, border = "grey60", col = "grey70", yaxt = "n",
	  ylim = c(0, 1), mgp = c(0, 0.2, 0), add = TRUE)
	
	mtext("Akaike Weights (AICc)", 2, line = 0.6)
	# text(bp[, 1], output.models[, 3] + 0.05, labels = c("BM", "OU", "EB", "timeSlice 66 Ma", "timeSlice 66-53 Ma", "release", "release & radiate", "release, radiate & recapture"), xpd = TRUE, srt = 0, cex = 0.75)

	text(bp[1:3, 1], output.models[1:3, 3] + 0.04, labels = c("BM", "OU", "EB"), xpd = TRUE, srt = 0, cex = 0.75)
	
	text(bp[4, 1], output.models[4, 3] + 0.1, labels = c("timeSlice"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[4, 1], output.models[4, 3] + 0.04, labels = c("66 Ma"), xpd = TRUE, srt = 0, cex = 0.75)
	
	text(bp[5, 1], output.models[5, 3] + 0.1, labels = c("timeSlice"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[5, 1], output.models[5, 3] + 0.04, labels = c("66 & 53 Ma"), xpd = TRUE, srt = 0, cex = 0.75)

	text(bp[6, 1], output.models[6, 3] + 0.16, labels = c("Naïve"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[6, 1], output.models[6, 3] + 0.1, labels = c("timeSlice"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[6, 1], output.models[6, 3] + 0.04, labels = c("171 Ma"), xpd = TRUE, srt = 0, cex = 0.75)
	
	text(bp[7, 1], output.models[7, 3] + 0.22, labels = c("modeSlice"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[7, 1], output.models[7, 3] + 0.16, labels = c("release"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[7, 1], output.models[7, 3] + 0.1, labels = c("OU to BM"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[7, 1], output.models[7, 3] + 0.04, labels = c("66 Ma"), xpd = TRUE, srt = 0, cex = 0.75)	
	
	
	text(bp[8, 1], output.models[8, 3] + 0.28, labels = c("modeSlice"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[8, 1], output.models[8, 3] + 0.22, labels = c("release & radiate"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[8, 1], output.models[8, 3] + 0.16, labels = c("OU to BM"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[8, 1], output.models[8, 3] + 0.1, labels = c("(BM rate change)"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[8, 1], output.models[8, 3] + 0.04, labels = c("66 Ma"), xpd = TRUE, srt = 0, cex = 0.75)	

	text(bp[9, 1], output.models[9, 3] + 0.28, labels = c("modeSlice"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[9, 1], output.models[9, 3] + 0.22, labels = c("release & radiate"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[9, 1], output.models[9, 3] + 0.16, labels = c("OU to BM to OU"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[9, 1], output.models[9, 3] + 0.1, labels = c("(BM rate change)"), xpd = TRUE, srt = 0, cex = 0.75)
	text(bp[9, 1], output.models[9, 3] + 0.04, labels = c("66 & 53 Ma"), xpd = TRUE, srt = 0, cex = 0.75)



