####################################################
## Code for simulations from Puttick et al. 2020  ##
####################################################


###########################
## Extant only simulations.
###########################


## Load motmot & other required packages.
library(motmot)
library(parallel)
library(TreeSim)

quiet <- function(x) { 
  sink(tempfile()) 
  on.exit(sink()) 
  invisible(force(x)) 
} 

# Function to run simulations on save files using the saveRDS function.
simulation.run <- function(ntips = 100, split.time = 0.5, timeRates = c(1, 1), n.splits = 3, name.file, mc.core = 1,
  boundaryAge = 0.05, testAge = 0.05, lambda = 1, mu = 0.5) {
  data.out <- mclapply(1:1000, mc.cores = mc.core, function(x) {
    phy <- sim.bd.taxa(n = ntips, numbsim = 1, lambda = lambda, mu = mu, frac = 1, complete = FALSE)[[1]]
	phy$edge.length <- phy$edge.length / (nodeTimes(phy)[1, 1])
	yd <- transformPhylo.sim(phy = phy, n = 1, model = "timeSlice", splitTime = split.time, timeRates = timeRates)
	yout <- quiet(transformPhylo.ML(yd, phy = phy, model="timeSlice", nSplits = n.splits, boundaryAge = boundaryAge, testAge = testAge))
	print(x)
	yout
	}
  )
  saveRDS(data.out, file = name.file)
}

## Number of cores (generally only usuable on mac & linux systems)
mc.core.n <- 1

### BM. Simultion of null models without shifts in rates. 
for(x in c(50, 100, 500))
  simulation.run(ntips = x, name.file = paste0("brownian.motion.", x, ".tips.Rds", mc.core = mc.core.n)

### Simulation of rate shifts from low rate to high rate at specified time. 
for(tip.x in c(50, 100, 500)) {
  for(time.x in c(0.1, 0.25, 0.5, 0.75, 0.9)) {
  	for(rate.x in c(2, 3, 10)) {
      simulation.run(ntips = x, split.time = 0.5, timeRates = c(1, rate.x), name.file = paste0("time.", time.x, "_rates.1.", rate.x, "_", tip.x, ".tips.Rds", mc.core = mc.core.n)
      }
    }
  }

### Simulation of rate shifts from high rate to low rate at specified time. 
for(tip.x in c(50, 100, 500)) {
  for(time.x in c(0.1, 0.25, 0.5, 0.75, 0.9)) {
  	for(rate.x in c(2, 3, 10)) {
      simulation.run(ntips = x, split.time = 0.5, timeRates = c(rate.x, 1), name.file = paste0("time.", time.x, "_rates.", rate.x, ".1_", tip.x, ".tips.Rds", mc.core = mc.core.n)
      }
    }
  }
  
  
######################################################
## Extant & Fossils Simualtions of Mammaliaformes tree
######################################################

## Load motmot.
library(motmot)

## Source mammaliaforme data & tree from the package.
data(mammals)
attach(mammals)

## Remove missing data, match data names to tree names, and take the columns 'mean' and 'sem' measurements from 'mammal.mass' matrix
trait.phy <- sortTraitData(phy = mammal.phy, y = as.matrix(mammal.mass), data.name = c("mean", "sem"), log.trait = FALSE)
mammal.phy <- trait.phy$phy
mammal.phy.extant <- drop.fossil(mammal.phy)
y <- as.matrix(trait.phy$trait[,1])
errors <- as.numeric(trait.phy$trait[,2])

full.nt <- nodeTimes(mammal.phy)
extant.nt <- nodeTimes(mammal.phy.extant)

quiet <- function(x) { 
  sink(tempfile()) 
  on.exit(sink()) 
  invisible(force(x)) 
} 

boundary.full <- c((max(full.nt)  - max(extant.nt)) + 10, 10)
boundary.extant <- c(10, 10)
testAge <- 55.76406
splits <- c(seq(max(extant.nt) - 10, 10, - testAge), 10)

# Function to run simulations on save files using the saveRDS function.
simulation.run <- function(phy, timeRates = c(1, 1), name.file, mc.core = 4, boundaryAge, testAge = 55.76406) {
  for(st in splits) {
    data.out <- parallel::mclapply(1:1000, mc.cores = mc.core, function(x) {
	  yd <- transformPhylo.sim(phy = phy, n = 1, model = "timeSlice", splitTime = st, timeRates = timeRates)
	  yout <- quiet(transformPhylo.ML(yd, phy = phy, model = "timeSlice", nSplits = 1, boundaryAge = boundaryAge, testAge = testAge))
	  print(x)
	  yout
	  }
	)
	name.out <- paste0(name.file, "_", signif(st, 3), ".Rds")
	saveRDS(data.out, file = name.out)
	}
  }


for(sim.n in c(2, 3, 5))
  simulation.run(mammal.phy, timeRates = c(sim.n, 1), name.file = paste0("full_", sim.n, ".1"), boundaryAge = boundary.full)

for(sim.n in c(2, 3, 5))
  simulation.run(mammal.phy, timeRates = c(1, sim.n), name.file = paste0("full_1.", sim.n), boundaryAge = boundary.full)

for(sim.n in c(2, 3, 5))
  simulation.run(mammal.phy.extant, timeRates = c(sim.n, 1), name.file = paste0("extant_", sim.n, ".1"), boundaryAge = boundary.extant)

for(sim.n in c(2, 3, 5))
  simulation.run(mammal.phy.extant, timeRates = c(sim.n, 2), name.file = paste0("extant_1.", sim.n), boundaryAge = boundary.extant)